<?php

defined( 'ABSPATH' ) or die;

$GLOBALS['processed_terms'] = array();
$GLOBALS['processed_posts'] = array();

require_once ABSPATH . 'wp-admin/includes/post.php';
require_once ABSPATH . 'wp-admin/includes/taxonomy.php';
require_once ABSPATH . 'wp-admin/includes/image.php';

/**
 * Add an Import Action, this data is stored for processing after import is done.
 *
 * Each action is sent as an Ajax request and is handled by themify-ajax.php file
 */ 
function themify_add_import_action( $action = '', $data = array() ) {
	global $import_actions;

	if ( ! isset( $import_actions[ $action ] ) ) {
		$import_actions[ $action ] = array();
	}

	$import_actions[ $action ][] = $data;
}

function themify_import_post( $post ) {
	global $processed_posts, $processed_terms;

	if ( ! post_type_exists( $post['post_type'] ) ) {
		return;
	}

	/* Menu items don't have reliable post_title, skip the post_exists check */
	if( $post['post_type'] !== 'nav_menu_item' ) {
		$post_exists = post_exists( $post['post_title'], '', $post['post_date'] );
		if ( $post_exists && get_post_type( $post_exists ) == $post['post_type'] ) {
			$processed_posts[ intval( $post['ID'] ) ] = intval( $post_exists );
			return;
		}
	}

	if( $post['post_type'] == 'nav_menu_item' ) {
		if( ! isset( $post['tax_input']['nav_menu'] ) || ! term_exists( $post['tax_input']['nav_menu'], 'nav_menu' ) ) {
			return;
		}
		$_menu_item_type = $post['meta_input']['_menu_item_type'];
		$_menu_item_object_id = $post['meta_input']['_menu_item_object_id'];

		if ( 'taxonomy' == $_menu_item_type && isset( $processed_terms[ intval( $_menu_item_object_id ) ] ) ) {
			$post['meta_input']['_menu_item_object_id'] = $processed_terms[ intval( $_menu_item_object_id ) ];
		} else if ( 'post_type' == $_menu_item_type && isset( $processed_posts[ intval( $_menu_item_object_id ) ] ) ) {
			$post['meta_input']['_menu_item_object_id'] = $processed_posts[ intval( $_menu_item_object_id ) ];
		} else if ( 'custom' != $_menu_item_type ) {
			// associated object is missing or not imported yet, we'll retry later
			// $missing_menu_items[] = $item;
			return;
		}
	}

	$post_parent = ( $post['post_type'] == 'nav_menu_item' ) ? $post['meta_input']['_menu_item_menu_item_parent'] : (int) $post['post_parent'];
	$post['post_parent'] = 0;
	if ( $post_parent ) {
		// if we already know the parent, map it to the new local ID
		if ( isset( $processed_posts[ $post_parent ] ) ) {
			if( $post['post_type'] == 'nav_menu_item' ) {
				$post['meta_input']['_menu_item_menu_item_parent'] = $processed_posts[ $post_parent ];
			} else {
				$post['post_parent'] = $processed_posts[ $post_parent ];
			}
		}
	}

	/**
	 * for hierarchical taxonomies, IDs must be used so wp_set_post_terms can function properly
	 * convert term slugs to IDs for hierarchical taxonomies
	 */
	if( ! empty( $post['tax_input'] ) ) {
		foreach( $post['tax_input'] as $tax => $terms ) {
			if( is_taxonomy_hierarchical( $tax ) ) {
				$terms = explode( ', ', $terms );
				$post['tax_input'][ $tax ] = array_map( 'themify_get_term_id_by_slug', $terms, array_fill( 0, count( $terms ), $tax ) );
			}
		}
	}

	$post['post_author'] = (int) get_current_user_id();
	$post['post_status'] = 'publish';

	$old_id = $post['ID'];

	unset( $post['ID'] );
	$post_id = wp_insert_post( $post, true );
	if( is_wp_error( $post_id ) ) {
		return false;
	} else {
		$processed_posts[ $old_id ] = $post_id;

		return $post_id;
	}
}

function themify_get_placeholder_image() {
	static $placeholder_image = null;

	if( $placeholder_image == null ) {
		if ( ! function_exists( 'WP_Filesystem' ) ) {
			require_once ABSPATH . 'wp-admin/includes/file.php';
		}
		WP_Filesystem();
		global $wp_filesystem;
		$upload = wp_upload_bits( $post['post_name'] . '.jpg', null, $wp_filesystem->get_contents( THEMIFY_DIR . '/img/image-placeholder.jpg' ) );

		if ( $info = wp_check_filetype( $upload['file'] ) )
			$post['post_mime_type'] = $info['type'];
		else
			return new WP_Error( 'attachment_processing_error', __( 'Invalid file type', 'themify' ) );

		$post['guid'] = $upload['url'];
		$post_id = wp_insert_attachment( $post, $upload['file'] );
		wp_update_attachment_metadata( $post_id, wp_generate_attachment_metadata( $post_id, $upload['file'] ) );

		$placeholder_image = $post_id;
	}

	return $placeholder_image;
}

function themify_import_term( $term ) {
	global $processed_terms;

	if( $term_id = term_exists( $term['slug'], $term['taxonomy'] ) ) {
		if ( is_array( $term_id ) ) $term_id = $term_id['term_id'];
		if ( isset( $term['term_id'] ) )
			$processed_terms[ intval( $term['term_id'] ) ] = (int) $term_id;
		return (int) $term_id;
	}

	if ( empty( $term['parent'] ) ) {
		$parent = 0;
	} else {
		$parent = term_exists( $processed_terms[ intval( $term['parent'] ) ], $term['taxonomy'] );
		if ( is_array( $parent ) ) $parent = $parent['term_id'];
	}

	$id = wp_insert_term( $term['name'], $term['taxonomy'], array(
		'parent' => $parent,
		'slug' => $term['slug'],
		'description' => $term['description'],
	) );
	if ( ! is_wp_error( $id ) ) {
		if ( isset( $term['term_id'] ) ) {
			// success!
			$processed_terms[ intval($term['term_id']) ] = $id['term_id'];
			if ( isset( $term['thumbnail'] ) ) {
				themify_add_import_action( 'term_thumb', array(
					'id' => $id['term_id'],
					'thumb' => $term['thumbnail'],
				) );
			}
			return $term['term_id'];
		}
	}

	return false;
}

function themify_get_term_id_by_slug( $slug, $tax ) {
	$term = get_term_by( 'slug', $slug, $tax );
	if( $term ) {
		return $term->term_id;
	}

	return false;
}

function themify_undo_import_term( $term ) {
	$term_id = term_exists( $term['slug'], $term['taxonomy'] );
	if ( $term_id ) {
		if ( is_array( $term_id ) ) $term_id = $term_id['term_id'];

		if ( $term_thumbnail = get_term_meta( $term['term_id'], 'thumbnail_id', true ) ) {
			wp_delete_attachment( $term_thumbnail, true );
		}

		if ( isset( $term_id ) ) {
			wp_delete_term( $term_id, $term['taxonomy'] );
		}
	}
}

/**
 * Determine if a post exists based on title, content, and date
 *
 * @global wpdb $wpdb WordPress database abstraction object.
 *
 * @param array $args array of database parameters to check
 * @return int Post ID if post exists, 0 otherwise.
 */
function themify_post_exists( $args = array() ) {
	global $wpdb;

	$query = "SELECT ID FROM $wpdb->posts WHERE 1=1";
	$db_args = array();

	foreach ( $args as $key => $value ) {
		$value = wp_unslash( sanitize_post_field( $key, $value, 0, 'db' ) );
		if( ! empty( $value ) ) {
			$query .= ' AND ' . $key . ' = %s';
			$db_args[] = $value;
		}
	}

	if ( !empty ( $args ) )
		return (int) $wpdb->get_var( $wpdb->prepare($query, $args) );

	return 0;
}

function themify_undo_import_post( $post ) {
	if( $post['post_type'] == 'nav_menu_item' ) {
		$post_exists = themify_post_exists( array(
			'post_name' => $post['post_name'],
			'post_modified' => $post['post_date'],
			'post_type' => 'nav_menu_item',
		) );
	} else {
		$post_exists = post_exists( $post['post_title'], '', $post['post_date'] );
	}
	if( $post_exists && get_post_type( $post_exists ) == $post['post_type'] ) {
		/**
		 * check if the post has been modified, if so leave it be
		 *
		 * NOTE: posts are imported using wp_insert_post() which modifies post_modified field
		 * to be the same as post_date, hence to check if the post has been modified,
		 * the post_modified field is compared against post_date in the original post.
		 */
		if( $post['post_date'] == get_post_field( 'post_modified', $post_exists ) ) {
			// find and remove all post attachments
			$attachments = get_posts( array(
				'post_type' => 'attachment',
				'posts_per_page' => -1,
				'post_parent' => $post_exists,
			) );
			if ( $attachments ) {
				foreach ( $attachments as $attachment ) {
					wp_delete_attachment( $attachment->ID, true );
				}
			}
			wp_delete_post( $post_exists, true ); // true: bypass trash
		}
	}
}

function themify_process_post_import( $post ) {
	if( ERASEDEMO ) {
		themify_undo_import_post( $post );
	} else {
		if ( $id = themify_import_post( $post ) ) {
			if ( defined( 'IMPORT_IMAGES' ) && ! IMPORT_IMAGES ) {
				/* if importing images is disabled and post is supposed to have a thumbnail, create a placeholder image instead */
				if ( isset( $post['thumb'] ) ) { // the post is supposed to have featured image
					$placeholder = themify_get_placeholder_image();
					if( ! is_wp_error( $placeholder ) ) {
						set_post_thumbnail( $id, $placeholder );
					}
				}
			} else {
				if ( isset( $post["thumb"] ) ) {
					themify_add_import_action( 'post_thumb', array(
						'id' => $id,
						'thumb' => $post["thumb"],
					) );
				}
				if ( isset( $post["gallery_shortcode"] ) ) {
					themify_add_import_action( 'gallery_field', array(
						'id' => $id,
						'fields' => $post["gallery_shortcode"],
					) );
				}
			}
		}
	}
}
$thumbs = array();
function themify_do_demo_import() {
global $import_actions;

	if ( isset( $GLOBALS["ThemifyBuilder_Data_Manager"] ) ) {
		remove_action( "save_post", array( $GLOBALS["ThemifyBuilder_Data_Manager"], "save_builder_text_only"), 10, 3 );
	}
$term = array (
  'term_id' => 1,
  'name' => 'Uncategorized',
  'slug' => 'uncategorized',
  'term_group' => 0,
  'taxonomy' => 'category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 3,
  'name' => 'Life',
  'slug' => 'life',
  'term_group' => 0,
  'taxonomy' => 'category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 4,
  'name' => 'Photos',
  'slug' => 'photos',
  'term_group' => 0,
  'taxonomy' => 'category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 5,
  'name' => 'Pets',
  'slug' => 'pets',
  'term_group' => 0,
  'taxonomy' => 'category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 6,
  'name' => 'Food',
  'slug' => 'food',
  'term_group' => 0,
  'taxonomy' => 'category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 7,
  'name' => 'Videos',
  'slug' => 'videos',
  'term_group' => 0,
  'taxonomy' => 'category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 8,
  'name' => 'Music',
  'slug' => 'music',
  'term_group' => 0,
  'taxonomy' => 'category',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 11,
  'name' => 'Pics',
  'slug' => 'pics',
  'term_group' => 0,
  'taxonomy' => 'post_tag',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$term = array (
  'term_id' => 12,
  'name' => 'embed',
  'slug' => 'embed',
  'term_group' => 0,
  'taxonomy' => 'post_tag',
  'description' => '',
  'parent' => 0,
);

if( ERASEDEMO ) {
	themify_undo_import_term( $term );
} else {
	themify_import_term( $term );
}

$post = array (
  'ID' => 4,
  'post_date' => '2012-03-20 19:27:33',
  'post_date_gmt' => '2012-03-20 19:27:33',
  'post_content' => 'Duis id tincidunt tortor. Curabitur placerat luctus lacinia. In hac habitasse platea dictumst. Suspendisse potenti. Nunc vestibulum, erat et pharetra aliquet, mi nunc iaculis erat, in pharetra libero felis sit amet ipsum.',
  'post_title' => 'Baking Cake',
  'post_excerpt' => '',
  'post_name' => 'baking-cake',
  'post_modified' => '2017-08-21 08:06:09',
  'post_modified_gmt' => '2017-08-21 08:06:09',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=4',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'life, photos',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/86531227.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 10,
  'post_date' => '2012-03-28 19:36:04',
  'post_date_gmt' => '2012-03-28 19:36:04',
  'post_content' => 'Etiam eget eleifend metus. Duis auctor orci et odio cursus adipiscing. Aliquam erat volutpat. In posuere faucibus justo, et faucibus lectus sodales ut. Duis auctor orci et odio cursus adipiscing.',
  'post_title' => 'Cute Puppy',
  'post_excerpt' => '',
  'post_name' => 'cute-puppy',
  'post_modified' => '2017-08-21 08:06:03',
  'post_modified_gmt' => '2017-08-21 08:06:03',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=10',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'pets',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/83454803.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 13,
  'post_date' => '2012-04-02 19:36:55',
  'post_date_gmt' => '2012-04-02 19:36:55',
  'post_content' => 'Vivamus in dolor eu lacus luctus auctor non ac turpis. Proin et rutrum dolor. Praesent venenatis purus convallis ipsum porttitor convallis consectetur orci condimentum. Curabitur ornare interdum pellentesque.',
  'post_title' => 'Picture From the Function',
  'post_excerpt' => '',
  'post_name' => 'picture-from-the-function',
  'post_modified' => '2017-08-21 08:05:53',
  'post_modified_gmt' => '2017-08-21 08:05:53',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=13',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'photos',
    'post_tag' => 'pics',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/83253659.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 16,
  'post_date' => '2012-04-17 19:37:52',
  'post_date_gmt' => '2012-04-17 19:37:52',
  'post_content' => 'Nullam nec elit sapien. Donec at congue sapien. Mauris a diam nec felis auctor ultrices. Sed tempus mollis orci in viverra. Vivamus libero eros, dictum id aliquam in, tristique id sapien.',
  'post_title' => 'Decorating a Cake',
  'post_excerpt' => '',
  'post_name' => 'decorating-a-cake',
  'post_modified' => '2017-08-21 08:05:51',
  'post_modified_gmt' => '2017-08-21 08:05:51',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=16',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'life',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/82088858.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 23,
  'post_date' => '2012-05-20 19:41:04',
  'post_date_gmt' => '2012-05-20 19:41:04',
  'post_content' => 'Praesent porta libero nec magna vulputate porta. Vivamus in dolor eu lacus luctus auctor non ac turpis. Proin et rutrum dolor.',
  'post_title' => 'Summer Picnic',
  'post_excerpt' => '',
  'post_name' => 'summer-picnic',
  'post_modified' => '2017-08-21 08:05:04',
  'post_modified_gmt' => '2017-08-21 08:05:04',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=23',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'life',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/86512918.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 44,
  'post_date' => '2012-05-20 19:49:29',
  'post_date_gmt' => '2012-05-20 19:49:29',
  'post_content' => 'In hac habitasse platea dictumst. Suspendisse potenti. Nunc vestibulum, erat et pharetra aliquet, mi nunc iaculis erat, in pharetra libero felis sit amet ipsum. Lorem ipsum dolor sit amet, consectetur adipiscing elit.',
  'post_title' => 'My New Home',
  'post_excerpt' => '',
  'post_name' => 'my-new-home',
  'post_modified' => '2017-08-21 08:05:02',
  'post_modified_gmt' => '2017-08-21 08:05:02',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=44',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'life',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/117971308.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 27,
  'post_date' => '2012-05-02 19:43:57',
  'post_date_gmt' => '2012-05-02 19:43:57',
  'post_content' => 'Cras ut urna est. Praesent eu ligula ut ligula tempus porttitor. In at sem ipsum, sed commodo felis. Nullam nec elit sapien. Donec at congue sapien. Mauris a diam nec felis auctor ultrices. Sed tempus mollis orci in viverra.',
  'post_title' => 'More Cake Photos',
  'post_excerpt' => '',
  'post_name' => 'more-cake-photos',
  'post_modified' => '2017-08-21 08:05:28',
  'post_modified_gmt' => '2017-08-21 08:05:28',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=27',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'life',
    'post_tag' => 'pics',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/86531137.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 30,
  'post_date' => '2012-04-20 19:44:36',
  'post_date_gmt' => '2012-04-20 19:44:36',
  'post_content' => 'Morbi varius eros neque, eu malesuada dolor. Etiam eget eleifend metus. Duis auctor orci et odio cursus adipiscing. Aliquam erat volutpat. In posuere faucibus justo, et faucibus lectus sodales ut.',
  'post_title' => 'New Picture Frame',
  'post_excerpt' => '',
  'post_name' => 'new-picture-frame',
  'post_modified' => '2017-08-21 08:05:49',
  'post_modified_gmt' => '2017-08-21 08:05:49',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=30',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'photos',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/86543180.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 33,
  'post_date' => '2012-04-29 19:45:56',
  'post_date_gmt' => '2012-04-29 19:45:56',
  'post_content' => 'Maecenas rhoncus malesuada aliquet. Morbi vulputate vulputate mauris quis condimentum. Ut ut ligula et mauris acc vulputate mauris quis cond the imentum umsan tristique ut nec lacus.',
  'post_title' => 'Surprise Office Gift',
  'post_excerpt' => '',
  'post_name' => 'surprise-office-gift',
  'post_modified' => '2017-08-21 08:05:37',
  'post_modified_gmt' => '2017-08-21 08:05:37',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=33',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'life',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/89795050.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 36,
  'post_date' => '2012-05-29 19:47:04',
  'post_date_gmt' => '2012-05-29 19:47:04',
  'post_content' => 'Duis auctor orci et odio cursus adipiscing. Aliquam erat volutpat. In posuere faucibus justo, et faucibus lectus sodales ut. Vestibulum bibendum, dolor sed auctor rhoncus, massa nulla faucibus dui, in sollicitudin elit est at lorem.',
  'post_title' => 'A Shot on the Lake',
  'post_excerpt' => '',
  'post_name' => 'a-shot-on-the-lake',
  'post_modified' => '2017-08-21 08:03:25',
  'post_modified_gmt' => '2017-08-21 08:03:25',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=36',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'photos',
    'post_tag' => 'pics',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/99387508.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 40,
  'post_date' => '2012-06-20 19:48:50',
  'post_date_gmt' => '2012-06-20 19:48:50',
  'post_content' => 'Pellentesque ornare, augue ac posuere consectetur, tellus ligula feugiat lorem, id imperdiet tellus tortor ut urna. Ut eleifend rhoncus augue sit amet dignissim. In mattis lobortis imperdiet. Aliquam molestie nisi et purus mattis volutpat. <!--more-->

Ut imperdiet, purus non pellentesque fringilla, erat est molestie nibh, sed tristique mi massa nec nisi. Cras ut urna est. Praesent eu ligula ut ligula tempus porttitor. In at sem ipsum, sed commodo felis. Nullam nec elit sapien. Donec at congue sapien. Mauris a diam nec felis auctor ultrices. Sed tempus mollis orci in viverra. Vivamus libero eros, dictum id aliquam in, tristique id sapien. Maecenas rhoncus malesuada aliquet. Morbi vulputate vulputate mauris quis condimentum. Ut ut ligula et mauris accumsan tristique ut nec lacus. Cras tempus pretium sagittis. Praesent porta libero nec magna vulputate porta. Vivamus in dolor eu lacus luctus auctor non ac turpis. Proin et rutrum dolor. Praesent venenatis purus convallis ipsum porttitor convallis consectetur orci condimentum. Curabitur ornare interdum pellentesque.',
  'post_title' => 'Super Model',
  'post_excerpt' => '',
  'post_name' => 'super-model',
  'post_modified' => '2017-08-21 08:02:45',
  'post_modified_gmt' => '2017-08-21 08:02:45',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=40',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
    'lightbox_link' => 'https://themify.me/demo/themes/pinboard/files/2012/06/108444018.jpg',
    'lightbox_icon' => 'on',
  ),
  'tax_input' => 
  array (
    'category' => 'photos',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/108444018.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 47,
  'post_date' => '2012-05-03 19:50:06',
  'post_date_gmt' => '2012-05-03 19:50:06',
  'post_content' => 'Pellentesque ornare, augue ac posuere consectetur, tellus ligula feugiat lorem, id imperdiet tellus tortor ut urna. Ut eleifend rhoncus augue sit amet dignissim.',
  'post_title' => 'Moi Moi',
  'post_excerpt' => '',
  'post_name' => 'moi-moi',
  'post_modified' => '2017-08-21 08:05:24',
  'post_modified_gmt' => '2017-08-21 08:05:24',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=47',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'pets',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/moi.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 50,
  'post_date' => '2012-05-03 19:52:14',
  'post_date_gmt' => '2012-05-03 19:52:14',
  'post_content' => 'Morbi varius eros neque, eu malesuada dolor. Etiam eget eleifend metus. Duis auctor orci et odio cursus adipiscing. Aliquam erat volutpat.',
  'post_title' => 'Rocky',
  'post_excerpt' => '',
  'post_name' => 'rocky',
  'post_modified' => '2017-08-21 08:05:21',
  'post_modified_gmt' => '2017-08-21 08:05:21',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=50',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'pets',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/rocky.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 54,
  'post_date' => '2012-06-02 19:53:39',
  'post_date_gmt' => '2012-06-02 19:53:39',
  'post_content' => 'Vivamus in dolor eu lacus luctus auctor non ac turpis. Proin et rutrum dolor. Praesent venenatis purus convallis ipsum porttitor convallis consectetur orci condimentum.',
  'post_title' => 'Two Models',
  'post_excerpt' => '',
  'post_name' => 'two-models',
  'post_modified' => '2017-08-21 08:03:12',
  'post_modified_gmt' => '2017-08-21 08:03:12',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=54',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'photos',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/124822261.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 60,
  'post_date' => '2012-04-20 19:59:11',
  'post_date_gmt' => '2012-04-20 19:59:11',
  'post_content' => 'Cras ut urna est. Praesent eu ligula ut ligula tempus porttitor. In at sem ipsum, sed commodo felis. Nullam nec elit sapien. Donec at congue sapien. Mauris a diam nec felis auctor ultrices.',
  'post_title' => 'Swinging',
  'post_excerpt' => '',
  'post_name' => 'swinging',
  'post_modified' => '2017-08-21 08:05:47',
  'post_modified_gmt' => '2017-08-21 08:05:47',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=60',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'life',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/124819326.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 63,
  'post_date' => '2012-06-17 20:01:53',
  'post_date_gmt' => '2012-06-17 20:01:53',
  'post_content' => 'Ut imperdiet, purus non pellentesque fringilla, erat est molestie nibh, sed tristique mi massa nec nisi.',
  'post_title' => 'Mm.. Cupcakes',
  'post_excerpt' => '',
  'post_name' => 'mm-cupcakes',
  'post_modified' => '2017-08-21 08:02:56',
  'post_modified_gmt' => '2017-08-21 08:02:56',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=63',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'food',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/141207092.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 66,
  'post_date' => '2012-06-20 20:02:48',
  'post_date_gmt' => '2012-06-20 20:02:48',
  'post_content' => 'Vivamus in dolor eu lacus luctus auctor non ac turpis. Proin et rutrum dolor. Praesent venenatis purus convallis ipsum porttitor convallis consectetur orci condimentum.',
  'post_title' => 'Lightbox Image',
  'post_excerpt' => '',
  'post_name' => 'lightbox-image',
  'post_modified' => '2017-08-21 08:02:44',
  'post_modified_gmt' => '2017-08-21 08:02:44',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=66',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
    'lightbox_link' => 'https://themify.me/demo/themes/pinboard/files/2012/06/133618178.jpg',
    'lightbox_icon' => 'on',
  ),
  'tax_input' => 
  array (
    'category' => 'photos',
    'post_tag' => 'pics',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/133618178.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 69,
  'post_date' => '2012-05-20 20:02:52',
  'post_date_gmt' => '2012-05-20 20:02:52',
  'post_content' => 'Vestibulum bibendum, dolor sed auctor rhoncus, massa nulla faucibus dui, in sollicitudin elit est at lorem. Proin aliquam gravida turpis et facilisis.',
  'post_title' => 'Photo Effect',
  'post_excerpt' => '',
  'post_name' => 'photo-effect',
  'post_modified' => '2017-08-21 08:04:58',
  'post_modified_gmt' => '2017-08-21 08:04:58',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=69',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'photos',
    'post_tag' => 'pics',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/135698842.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 72,
  'post_date' => '2012-05-28 20:03:44',
  'post_date_gmt' => '2012-05-28 20:03:44',
  'post_content' => 'Praesent eu ligula ut ligula tempus porttitor. In at sem ipsum, sed commodo felis. Nullam nec elit sapien. Donec at congue sapien.',
  'post_title' => 'Just Two Kids',
  'post_excerpt' => '',
  'post_name' => 'just-two-kids',
  'post_modified' => '2017-08-21 08:03:26',
  'post_modified_gmt' => '2017-08-21 08:03:26',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=72',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'life',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/136295491.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 75,
  'post_date' => '2012-06-03 20:04:19',
  'post_date_gmt' => '2012-06-03 20:04:19',
  'post_content' => 'Morbi varius eros neque, eu malesuada dolor. Etiam eget eleifend metus.',
  'post_title' => 'Lovely',
  'post_excerpt' => '',
  'post_name' => 'lovely',
  'post_modified' => '2017-08-21 08:03:03',
  'post_modified_gmt' => '2017-08-21 08:03:03',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=75',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'life',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/140788124.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 78,
  'post_date' => '2012-05-06 20:05:06',
  'post_date_gmt' => '2012-05-06 20:05:06',
  'post_content' => 'Vivamus in dolor eu lacus luctus auctor non ac turpis. Proin et rutrum dolor. Praesent venenatis purus convallis ipsum porttitor convallis consectetur orci condimentum. Curabitur ornare interdum pellentesque. <!--more-->

Ut imperdiet, purus non pellentesque fringilla, erat est molestie nibh, sed tristique mi massa nec nisi. Cras ut urna est. Praesent eu ligula ut ligula tempus porttitor. In at sem ipsum, sed commodo felis. Nullam nec elit sapien. Donec at congue sapien. Mauris a diam nec felis auctor ultrices. Sed tempus mollis orci in viverra. Vivamus libero eros, dictum id aliquam in, tristique id sapien. Maecenas rhoncus malesuada aliquet. Morbi vulputate vulputate mauris quis condimentum. Ut ut ligula et mauris accumsan tristique ut nec lacus. Cras tempus pretium sagittis. Praesent porta libero nec magna vulputate porta. Vivamus in dolor eu lacus luctus auctor non ac turpis. Proin et rutrum dolor. Praesent venenatis purus convallis ipsum porttitor convallis consectetur orci condimentum. Curabitur ornare interdum pellentesque.',
  'post_title' => 'A Travel Photo',
  'post_excerpt' => '',
  'post_name' => 'a-travel-photo',
  'post_modified' => '2017-08-21 08:05:12',
  'post_modified_gmt' => '2017-08-21 08:05:12',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=78',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'photos',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/200471810-001.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 81,
  'post_date' => '2012-06-20 20:08:06',
  'post_date_gmt' => '2012-06-20 20:08:06',
  'post_content' => 'Ut imperdiet, purus non pellentesque fringilla, erat est molestie nibh, sed tristique mi massa nec nisi. Cras ut urna est. Praesent eu ligula ut ligula tempus porttitor. In at sem ipsum, sed commodo felis. Nullam nec elit sapien. Donec at congue sapien. Mauris a diam nec felis auctor ultrices. <!--more-->Sed tempus mollis orci in viverra. Vivamus libero eros, dictum id aliquam in, tristique id sapien. Maecenas rhoncus malesuada aliquet. Morbi vulputate vulputate mauris quis condimentum. Ut ut ligula et mauris accumsan tristique ut nec lacus. Cras tempus pretium sagittis. Praesent porta libero nec magna vulputate porta. Vivamus in dolor eu lacus luctus auctor non ac turpis. Proin et rutrum dolor. Praesent venenatis purus convallis ipsum porttitor convallis consectetur orci condimentum. Curabitur ornare interdum pellentesque.',
  'post_title' => 'Lightbox YouTube Video',
  'post_excerpt' => '',
  'post_name' => 'lightbox-youtube-video',
  'post_modified' => '2017-08-21 08:02:42',
  'post_modified_gmt' => '2017-08-21 08:02:42',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=81',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
    'lightbox_link' => 'http://www.youtube.com/watch?v=NmRTreaCJXs&wmode=transparent',
    'lightbox_icon' => 'on',
  ),
  'tax_input' => 
  array (
    'category' => 'videos',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/78159291.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 85,
  'post_date' => '2012-06-20 17:08:41',
  'post_date_gmt' => '2012-06-20 17:08:41',
  'post_content' => 'Praesent venenatis purus convallis ipsum porttitor convallis consectetur orci condimentum. Curabitur ornare interdum pellentesque.',
  'post_title' => 'Vimeo Video Embed',
  'post_excerpt' => '',
  'post_name' => 'vimeo-video-embed',
  'post_modified' => '2017-08-21 08:02:47',
  'post_modified_gmt' => '2017-08-21 08:02:47',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=85',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
    'video_url' => 'http://vimeo.com/6284199',
  ),
  'tax_input' => 
  array (
    'category' => 'videos',
    'post_tag' => 'embed',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 89,
  'post_date' => '2012-06-20 16:10:38',
  'post_date_gmt' => '2012-06-20 16:10:38',
  'post_content' => 'Vivamus in dolor eu lacus luctus auctor non ac turpis. Proin et rutrum dolor.',
  'post_title' => 'YouTube Video',
  'post_excerpt' => '',
  'post_name' => 'youtube-video',
  'post_modified' => '2017-08-21 08:06:20',
  'post_modified_gmt' => '2017-08-21 08:06:20',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=89',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
    'video_url' => 'http://youtu.be/Abjx1JJO1i8',
  ),
  'tax_input' => 
  array (
    'category' => 'videos',
    'post_tag' => 'embed',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 91,
  'post_date' => '2012-06-19 20:11:37',
  'post_date_gmt' => '2012-06-19 20:11:37',
  'post_content' => 'Cras tempus pretium sagittis. Praesent porta libero nec magna vulputate porta.',
  'post_title' => 'Light iFrame Window',
  'post_excerpt' => '',
  'post_name' => 'light-iframe-window',
  'post_modified' => '2017-08-21 08:02:50',
  'post_modified_gmt' => '2017-08-21 08:02:50',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=91',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
    'lightbox_link' => 'https://themify.me?iframe=true&width=100%&height=100%',
  ),
  'tax_input' => 
  array (
    'category' => 'life',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/137155350.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 96,
  'post_date' => '2012-04-30 20:14:28',
  'post_date_gmt' => '2012-04-30 20:14:28',
  'post_content' => 'Vivamus in dolor eu lacus luctus auctor non ac turpis. Proin et rutrum dolor. Praesent venenatis purus convallis ipsum porttitor convallis consectetur orci condimentum. Curabitur ornare interdum pellentesque.',
  'post_title' => 'Grill Shrimps',
  'post_excerpt' => '',
  'post_name' => 'grill-shrimps',
  'post_modified' => '2017-08-21 08:05:35',
  'post_modified_gmt' => '2017-08-21 08:05:35',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=96',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'photos',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/105824452.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 99,
  'post_date' => '2012-03-29 20:15:51',
  'post_date_gmt' => '2012-03-29 20:15:51',
  'post_content' => 'Proin et rutrum dolor. Praesent venenatis purus convallis ipsum porttitor convallis consectetur orci condimentum. Curabitur ornare interdum pellentesque.',
  'post_title' => 'Super Platter',
  'post_excerpt' => '',
  'post_name' => 'super-platter',
  'post_modified' => '2017-08-21 08:05:57',
  'post_modified_gmt' => '2017-08-21 08:05:57',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=99',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'photos',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/105674282.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 102,
  'post_date' => '2012-06-01 20:16:36',
  'post_date_gmt' => '2012-06-01 20:16:36',
  'post_content' => 'Ut ut ligula et mauris accumsan tristique ut nec lacus. Cras tempus pretium sagittis. Praesent porta libero nec magna vulputate porta.',
  'post_title' => 'Hot DJ',
  'post_excerpt' => '',
  'post_name' => 'hot-dj',
  'post_modified' => '2017-08-21 08:03:24',
  'post_modified_gmt' => '2017-08-21 08:03:24',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=102',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'music',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/106413309.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 105,
  'post_date' => '2012-06-01 20:18:00',
  'post_date_gmt' => '2012-06-01 20:18:00',
  'post_content' => 'Praesent venenatis purus convallis ipsum porttitor convallis consectetur orci condimentum. Curabitur ornare interdum pellentesque.',
  'post_title' => 'Japanese Model',
  'post_excerpt' => '',
  'post_name' => 'japanese-model',
  'post_modified' => '2017-08-21 08:03:15',
  'post_modified_gmt' => '2017-08-21 08:03:15',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=105',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
    'lightbox_link' => 'https://themify.me/demo/themes/pinboard/files/2012/06/104257717.jpg',
  ),
  'tax_input' => 
  array (
    'category' => 'photos',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/104257717.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 109,
  'post_date' => '2012-04-20 20:18:45',
  'post_date_gmt' => '2012-04-20 20:18:45',
  'post_content' => 'Vivamus in dolor eu lacus luctus auctor non ac turpis. Proin et rutrum dolor.',
  'post_title' => 'New Album',
  'post_excerpt' => '',
  'post_name' => 'new-album',
  'post_modified' => '2017-08-21 08:05:46',
  'post_modified_gmt' => '2017-08-21 08:05:46',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=109',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'music',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/92376054.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 112,
  'post_date' => '2012-05-27 20:19:32',
  'post_date_gmt' => '2012-05-27 20:19:32',
  'post_content' => 'Ut imperdiet, purus non pellentesque fringilla, erat est molestie nibh, sed tristique mi massa nec nisi. Cras ut urna est.',
  'post_title' => 'Rock Band Concert',
  'post_excerpt' => '',
  'post_name' => 'rock-band-concert',
  'post_modified' => '2017-08-21 08:04:56',
  'post_modified_gmt' => '2017-08-21 08:04:56',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=112',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'music',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/105547706.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 115,
  'post_date' => '2012-05-03 20:20:41',
  'post_date_gmt' => '2012-05-03 20:20:41',
  'post_content' => 'Ut urus non pellentesque fringilla, erat est molestie nibh, sed tristique mi massa nec nisi. Cras ut urna est.',
  'post_title' => 'Masked Party',
  'post_excerpt' => '',
  'post_name' => 'masked-party',
  'post_modified' => '2017-08-21 08:05:18',
  'post_modified_gmt' => '2017-08-21 08:05:18',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=115',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'music',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/sb10068474al-001.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 118,
  'post_date' => '2012-05-27 20:21:33',
  'post_date_gmt' => '2012-05-27 20:21:33',
  'post_content' => 'Duis dignissim mollis lectus, id mollis tortor dignissim ut. Proin non velit arcu, sit amet laoreet massa.',
  'post_title' => 'Soccer Player',
  'post_excerpt' => '',
  'post_name' => 'soccer-player',
  'post_modified' => '2017-08-21 08:03:27',
  'post_modified_gmt' => '2017-08-21 08:03:27',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=118',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'life',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/sb10063098cd-001.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 121,
  'post_date' => '2012-03-28 20:22:34',
  'post_date_gmt' => '2012-03-28 20:22:34',
  'post_content' => 'Aliquam gravida eros sit amet leo scelerisque molestie. Morbi et cursus felis.',
  'post_title' => 'Camera Photo',
  'post_excerpt' => '',
  'post_name' => 'camera-photo',
  'post_modified' => '2017-08-21 08:06:01',
  'post_modified_gmt' => '2017-08-21 08:06:01',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=121',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'photos',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/114407749.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 124,
  'post_date' => '2012-03-20 20:23:27',
  'post_date_gmt' => '2012-03-20 20:23:27',
  'post_content' => 'Integer euismod tincidunt nisl, in iaculis tellus feugiat sed. Praesent semper augue eu augue ullamcorper mattis.',
  'post_title' => 'Speaking',
  'post_excerpt' => '',
  'post_name' => 'speaking',
  'post_modified' => '2017-08-21 08:06:07',
  'post_modified_gmt' => '2017-08-21 08:06:07',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=124',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'life',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/89792997.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 129,
  'post_date' => '2012-06-01 20:30:01',
  'post_date_gmt' => '2012-06-01 20:30:01',
  'post_content' => 'Maecenas rhoncus malesuada aliquet. Morbi vulputate vulputate mauris quis condimentum.',
  'post_title' => 'Vintage Red',
  'post_excerpt' => '',
  'post_name' => 'vintage-red',
  'post_modified' => '2017-08-21 08:03:15',
  'post_modified_gmt' => '2017-08-21 08:03:15',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=129',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'photos',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/101793848.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 132,
  'post_date' => '2012-03-28 20:35:10',
  'post_date_gmt' => '2012-03-28 20:35:10',
  'post_content' => 'Sed tempus mollis orci in viverra. Vivamus libero eros, dictum id aliquam in, tristique id sapien.',
  'post_title' => 'Hi Cuties',
  'post_excerpt' => '',
  'post_name' => 'hi-cuties',
  'post_modified' => '2017-08-21 08:05:59',
  'post_modified_gmt' => '2017-08-21 08:05:59',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=132',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'photos',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/119860909.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 136,
  'post_date' => '2012-06-10 20:37:00',
  'post_date_gmt' => '2012-06-10 20:37:00',
  'post_content' => 'Praesent eu ligula ut ligula tempus porttitor. In at sem ipsum, sed commodo felis. Nullam nec elit sapien.',
  'post_title' => 'In the Car',
  'post_excerpt' => '',
  'post_name' => 'in-the-car',
  'post_modified' => '2017-08-21 08:02:59',
  'post_modified_gmt' => '2017-08-21 08:02:59',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=136',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
    'lightbox_link' => 'https://themify.me/demo/themes/pinboard/files/2012/06/200285694-001.jpg',
  ),
  'tax_input' => 
  array (
    'category' => 'uncategorized',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/200285694-001.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 139,
  'post_date' => '2012-06-20 20:40:06',
  'post_date_gmt' => '2012-06-20 20:40:06',
  'post_content' => 'Praesent venenatis purus convallis ipsum porttitor convallis consectetur orci condimentum. Curabitur ornare interdum pellentesque.

[gallery link="file" columns="1" order="DESC"]

Another theme by <a href="https://themify.me">Themify.me</a>',
  'post_title' => 'Photo Gallery',
  'post_excerpt' => '',
  'post_name' => 'photo-gallery',
  'post_modified' => '2017-08-21 08:02:40',
  'post_modified_gmt' => '2017-08-21 08:02:40',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=139',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
    'category' => 'photos',
    'post_tag' => 'pics',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/121375483.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 152,
  'post_date' => '2012-06-14 20:41:10',
  'post_date_gmt' => '2012-06-14 20:41:10',
  'post_content' => 'Proin aliquam gravida turpis et facilisis. Sed ornare enim et odio dictum dapibus ac vitae dolor.

[gallery link="file" columns="1" order="DESC"]',
  'post_title' => 'B & W Gallery',
  'post_excerpt' => '',
  'post_name' => 'b-w-gallery',
  'post_modified' => '2017-08-21 08:02:58',
  'post_modified_gmt' => '2017-08-21 08:02:58',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=152',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'photos',
    'post_tag' => 'pics',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/92052891.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 161,
  'post_date' => '2012-05-05 21:58:24',
  'post_date_gmt' => '2012-05-05 21:58:24',
  'post_content' => 'Magna pulchra canis in tertia planeta a sole. Sed augue lacus, porta eu sollicitudin vel, molestie et lacus. In sit amet turpis id velit laoreet venenatis non eget diam.',
  'post_title' => 'Dango',
  'post_excerpt' => '',
  'post_name' => 'dango',
  'post_modified' => '2017-08-21 08:05:13',
  'post_modified_gmt' => '2017-08-21 08:05:13',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=161',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'pets',
    'post_tag' => 'pics',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/dango.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 202,
  'post_date' => '2012-05-04 22:15:03',
  'post_date_gmt' => '2012-05-04 22:15:03',
  'post_content' => 'Cras ut urna est. Praesent eu ligula ut ligula tempus porttitor. In at sem ipsum, sed commodo felis. Nullam nec elit sapien. Donec at congue sapien. Mauris a diam nec felis auctor ultrices. Sed tempus mollis orci in viverra.',
  'post_title' => 'My Boy',
  'post_excerpt' => '',
  'post_name' => 'my-boy',
  'post_modified' => '2017-08-21 08:05:16',
  'post_modified_gmt' => '2017-08-21 08:05:16',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=202',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'life, photos',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/jayden.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 217,
  'post_date' => '2012-06-01 23:18:44',
  'post_date_gmt' => '2012-06-01 23:18:44',
  'post_content' => 'Vivamus in dolor eu lacus luctus auctor non ac turpis. Proin et rutrum dolor. Praesent venenatis purus convallis ipsum porttitor convallis consectetur orci condimentum. Curabitur ornare interdum pellentesque.',
  'post_title' => 'A Video Embed',
  'post_excerpt' => '',
  'post_name' => 'a-video-embed',
  'post_modified' => '2017-08-21 08:03:14',
  'post_modified_gmt' => '2017-08-21 08:03:14',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=217',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
    'video_url' => 'http://vimeo.com/43797471',
  ),
  'tax_input' => 
  array (
    'category' => 'videos',
    'post_tag' => 'embed',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 220,
  'post_date' => '2012-06-02 23:21:01',
  'post_date_gmt' => '2012-06-02 23:21:01',
  'post_content' => '[gallery link="file" columns="1" order="DESC"]

Ut imperdiet, purus non pellentesque fringilla, erat est molestie nibh, sed tristique mi massa nec nisi.<!--more-->

Cras ut urna est. Praesent eu ligula ut ligula tempus porttitor. In at sem ipsum, sed commodo felis. Nullam nec elit sapien. Donec at congue sapien. Mauris a diam nec felis auctor ultrices. Sed tempus mollis orci in viverra. Vivamus libero eros, dictum id aliquam in, tristique id sapien. Maecenas rhoncus malesuada aliquet. Morbi vulputate vulputate mauris quis condimentum. Ut ut ligula et mauris accumsan tristique ut nec lacus. Cras tempus pretium sagittis. Praesent porta libero nec magna vulputate porta. Vivamus in dolor eu lacus luctus auctor non ac turpis. Proin et rutrum dolor. Praesent venenatis purus convallis ipsum porttitor convallis consectetur orci condimentum. Curabitur ornare interdum pellentesque.',
  'post_title' => 'Dark Photo Effect',
  'post_excerpt' => '',
  'post_name' => 'dark-photo-effect',
  'post_modified' => '2017-08-21 08:03:06',
  'post_modified_gmt' => '2017-08-21 08:03:06',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=220',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'photos',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/91772695.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 234,
  'post_date' => '2012-04-21 16:13:58',
  'post_date_gmt' => '2012-04-21 16:13:58',
  'post_content' => 'Praesent venenatis purus convallis ipsum porttitor convallis consectetur orci condimentum. Curabitur ornare interdum pellentesque.',
  'post_title' => 'Shopping Time',
  'post_excerpt' => '',
  'post_name' => 'shopping-time',
  'post_modified' => '2017-08-21 08:05:42',
  'post_modified_gmt' => '2017-08-21 08:05:42',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=234',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'life',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/86503538.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 237,
  'post_date' => '2012-03-21 16:17:15',
  'post_date_gmt' => '2012-03-21 16:17:15',
  'post_content' => 'Praesent eu ligula ut ligula tempus porttitor. In at sem ipsum, sed commodo felis. Nullam nec elit sapien. Donec at congue sapien. Mauris a diam nec felis auctor ultrices.<!--more-->

Sed tempus mollis orci in viverra. Vivamus libero eros, dictum id aliquam in, tristique id sapien. Maecenas rhoncus malesuada aliquet. Morbi vulputate vulputate mauris quis condimentum. Ut ut ligula et mauris accumsan tristique ut nec lacus. Cras tempus pretium sagittis. Praesent porta libero nec magna vulputate porta. Vivamus in dolor eu lacus luctus auctor non ac turpis. Proin et rutrum dolor. Praesent venenatis purus convallis ipsum porttitor convallis consectetur orci condimentum. Curabitur ornare interdum pellentesque.',
  'post_title' => 'Lightbox Vimeo',
  'post_excerpt' => '',
  'post_name' => 'lightbox-vimeo',
  'post_modified' => '2017-08-21 08:06:06',
  'post_modified_gmt' => '2017-08-21 08:06:06',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=237',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
    'lightbox_link' => 'http://vimeo.com/43797471',
  ),
  'tax_input' => 
  array (
    'category' => 'videos',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/86481397.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 243,
  'post_date' => '2012-03-03 17:48:54',
  'post_date_gmt' => '2012-03-03 17:48:54',
  'post_content' => 'Praesent venenatis purus convallis ipsum porttitor convallis consectetur orci condimentum. Curabitur ornare interdum pellentesque.',
  'post_title' => 'Blue Heaven Video',
  'post_excerpt' => '',
  'post_name' => 'blue-heaven-video',
  'post_modified' => '2017-08-21 08:06:13',
  'post_modified_gmt' => '2017-08-21 08:06:13',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=243',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
    'video_url' => 'http://vimeo.com/44253174',
  ),
  'tax_input' => 
  array (
    'category' => 'videos',
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 245,
  'post_date' => '2012-03-06 17:49:38',
  'post_date_gmt' => '2012-03-06 17:49:38',
  'post_content' => 'Praesent porta libero nec magna vulputate porta. Vivamus in dolor eu lacus luctus auctor non ac turpis. Proin et rutrum dolor.',
  'post_title' => 'Sunset Image',
  'post_excerpt' => '',
  'post_name' => 'sunset-image',
  'post_modified' => '2017-08-21 08:06:16',
  'post_modified_gmt' => '2017-08-21 08:06:16',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?p=245',
  'menu_order' => 0,
  'post_type' => 'post',
  'meta_input' => 
  array (
    'content_width' => 'default_width',
    'layout' => 'default',
    'hide_post_title' => 'default',
    'unlink_post_title' => 'default',
    'hide_post_meta' => 'default',
    'hide_post_date' => 'default',
    'hide_post_image' => 'default',
    'unlink_post_image' => 'default',
  ),
  'tax_input' => 
  array (
    'category' => 'photos',
  ),
  'thumb' => 'https://themify.me/demo/themes/pinboard/files/2012/06/86529817.jpg',
);
themify_process_post_import( $post );


$post = array (
  'ID' => 162,
  'post_date' => '2012-06-20 21:35:52',
  'post_date_gmt' => '2012-06-20 21:35:52',
  'post_content' => '<h3>Buttons</h3>
[button style="orange" link="https://themify.me"]Orange[/button] [button style="blue"]Blue[/button] [button style="pink"]Pink[/button] [button style="green"]Green[/button] [button style="red"]Red[/button] [button style="black"]Black[/button]

[hr]

[button style="small"]Small[/button]

[button]Default[/button]

[button style="large"]Large[/button] [button style="xlarge"]Xlarge[/button]

[hr]

[button style="orange small"]Orange Small[/button] [button style="blue"]Blue[/button] [button style="green large"]Green Large[/button] [button style="red xlarge"]Red Xlarge[/button]

[hr]
<h3>Columns</h3>
[col grid="2-1 first"]
<h4>col 2-1</h4>
Sed sagittis, elit egestas rutrum vehicula, neque dolor fringilla lacus, ut rhoncus turpis augue vitae libero. Nam risus velit, rhoncus eg.

[/col]

[col grid="2-1"]
<h4>col 2-1</h4>
Curabitur vel risus eros, sed eleifend arcu. Donec porttitor hendrerit diam et blandit. Curabitur vitae velit ligula, vitae lobortis massa.

[/col]

[hr]

[col grid="3-1 first"]
<h4>col 3-1</h4>
Sed sagittis, elit egestas rutrum vehicula, neque dolor fringilla lacus, ut rhoncus turpis augue vitae libero. Nam risus velit, rhoncus eg.

[/col]

[col grid="3-1"]
<h4>col 3-1</h4>
Curabitur vel risus eros, sed eleifend arcu. Donec porttitor hendrerit diam et blandit. Curabitur vitae velit ligula, vitae lobortis massa.

[/col]

[col grid="3-1"]
<h4>col 3-1</h4>
Vivamus dignissim, ligula velt pretium leo, vel placerat ipsum risus luctus purus. Tos, sed eleifend arcu. Donec porttitor hendrerit.

[/col]

[hr]

[col grid="4-1 first"]
<h4>col 4-1</h4>
Sed sagittis, elit egestas rutrum vehicula, neque dolor fringilla lacus, ut rhoncus turpis augue vitae libero. Nam risus velit, rhoncus eget co.

[/col]

[col grid="4-1"]
<h4>col 4-1</h4>
Curabitur vel risus eros, sed eleifend arcu. Donec porttitor hendrerit diam et blandit. Curabitur vitae velit ligula, vitae lobortis mas.

[/col]

[col grid="4-1"]
<h4>col 4-1</h4>
Vivamus dignissim, ligula velt pretium leo, vel placerat ipsum risus luctus purus. Tos, sed eleifend arcu. Donec porttitor hendrerit diam.

[/col]

[col grid="4-1"]
<h4>col 4-1</h4>
Donec porttitor hendrerit diam et blandit. Curabitur vel risus eros, sed eleifend arcu. Curabitur vitae velit ligula, vitae lobortis mas.

[/col]

[hr]

[col grid="4-2 first"]
<h4>col 4-2</h4>
Sed sagittis, elit egestas rutrum vehicula, neque dolor fringilla lacus, ut rhoncus turpis augue vitae libero. Nam risus velit, rhoncus eget cout rhoncus turpis augue vitae libero.

[/col]

[col grid="4-1"]
<h4>col 4-1</h4>
Curabitur vel risus eros, sed eleifend arcu. Donec porttitor hendrerit diam et blandit. Curabitur vitae velit ligula, vitae lobortis mas.

[/col]

[col grid="4-1"]
<h4>col 4-1</h4>
Vivamus dignissim, ligula velt pretium leo, vel placerat ipsum risus luctus purus. Tos, sed eleifend arcu. Donec porttitor hendrerit diam.

[/col]
<h3>Horizontal Rules</h3>
[hr]

[hr color="pink"]

[hr color="red"]

[hr color="light-gray"]

[hr color="dark-gray"]

[hr color="black"]

[hr color="orange"]

[hr color="yellow"]

[hr color="white"]
<h3>Quote</h3>
[quote]Vivamus in risus non lacus vehicula vestibulum. In magna leo, malesuada eget pulvinar ut, pellentesque a arcu. Praesent rutrum feugiat nibh elementum posuere. Nulla volutpat porta enim vel consectetur. Etiam orci eros, blandit nec egestas eget, pharetra eget leo. Morbi lobortis adipiscing massa tincidunt dignissim. Nulla lobortis laoreet risus, tempor accumsan sem congue vitae. Cras laoreet hendrerit erat, id porttitor nunc blandit adipiscing. [/quote]
<h3>Map</h3>
[map address="Yonge St. and Eglinton Ave, Toronto, Ontario, Canada" width=100% height=400px]',
  'post_title' => 'Shortcodes',
  'post_excerpt' => '',
  'post_name' => 'shortcodes',
  'post_modified' => '2017-08-21 08:08:03',
  'post_modified_gmt' => '2017-08-21 08:08:03',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?page_id=162',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_width' => 'default_width',
    'hide_page_title' => 'default',
    'order' => 'desc',
    'orderby' => 'date',
    'post_in_lightbox' => 'default',
    'display_content' => 'content',
    'hide_title' => 'default',
    'unlink_title' => 'default',
    'hide_date' => 'default',
    'hide_meta' => 'default',
    'hide_image' => 'default',
    'unlink_image' => 'default',
    'hide_navigation' => 'default',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 169,
  'post_date' => '2012-06-20 21:39:39',
  'post_date_gmt' => '2012-06-20 21:39:39',
  'post_content' => '[col grid="3-2 first"]
[map address="Yonge St. and Eglinton Ave, Toronto, Ontario, Canada" width=100% height=600px]
[/col]

[col grid="3-1"]
<h3>Direction</h3>
We are located at Aliquam faucibus turpis at libero consectetur euismod. Nam nunc lectus, congue non egestas quis, condimentum ut arcu. Nulla placerat, tortor non egestas rutrum, mi turpis adipiscing dui, et mollis turpis tortor vel orci. Cras a fringilla nunc. Suspendisse volutpat, eros congue scelerisque iaculis, magna odio sodales dui, vitae vulputate elit metus ac arcu.
<h3>Address</h3>
123 Street Name,
City, Province
23446
<h3>Phone</h3>
236-298-2828
<h3>Hours</h3>
Mon - Fri : 11:00am - 10:00pm
Sat : 11:00am - 2:00pm
Sun : 12:00am - 11:00pm
[/col]',
  'post_title' => 'Map',
  'post_excerpt' => '',
  'post_name' => 'map',
  'post_modified' => '2017-08-21 08:08:01',
  'post_modified_gmt' => '2017-08-21 08:08:01',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?page_id=169',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_width' => 'default_width',
    'hide_page_title' => 'default',
    'order' => 'desc',
    'orderby' => 'date',
    'post_in_lightbox' => 'default',
    'display_content' => 'content',
    'hide_title' => 'default',
    'unlink_title' => 'default',
    'hide_date' => 'default',
    'hide_meta' => 'default',
    'hide_image' => 'default',
    'unlink_image' => 'default',
    'hide_navigation' => 'default',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 171,
  'post_date' => '2012-06-20 21:39:59',
  'post_date_gmt' => '2012-06-20 21:39:59',
  'post_content' => 'Download this contact plugin: <a href="http://wordpress.org/extend/plugins/contact-form-7">Contact Form 7</a>.

[contact-form-7 id="174" title="Untitled"]',
  'post_title' => 'Contact',
  'post_excerpt' => '',
  'post_name' => 'contact',
  'post_modified' => '2017-08-21 08:07:47',
  'post_modified_gmt' => '2017-08-21 08:07:47',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?page_id=171',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'default',
    'content_width' => 'default_width',
    'hide_page_title' => 'default',
    'order' => 'desc',
    'orderby' => 'date',
    'post_in_lightbox' => 'default',
    'display_content' => 'content',
    'hide_title' => 'default',
    'unlink_title' => 'default',
    'hide_date' => 'default',
    'hide_meta' => 'default',
    'hide_image' => 'default',
    'unlink_image' => 'default',
    'hide_navigation' => 'default',
    'builder_switch_frontend' => '0',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 279,
  'post_date' => '2012-11-29 19:36:04',
  'post_date_gmt' => '2012-11-29 19:36:04',
  'post_content' => '',
  'post_title' => 'With Sidebar',
  'post_excerpt' => '',
  'post_name' => 'with-sidebar',
  'post_modified' => '2017-10-28 00:28:08',
  'post_modified_gmt' => '2017-10-28 00:28:08',
  'post_content_filtered' => '',
  'post_parent' => 178,
  'guid' => 'https://themify.me/demo/themes/pinboard/?page_id=279',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar1',
    'content_width' => 'default_width',
    'hide_page_title' => 'default',
    'query_category' => '0',
    'order' => 'desc',
    'orderby' => 'date',
    'post_in_lightbox' => 'default',
    'display_content' => 'content',
    'image_width' => '222',
    'hide_title' => 'default',
    'unlink_title' => 'default',
    'hide_date' => 'default',
    'hide_meta' => 'default',
    'hide_image' => 'default',
    'unlink_image' => 'default',
    'hide_navigation' => 'default',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 178,
  'post_date' => '2012-06-20 21:45:07',
  'post_date_gmt' => '2012-06-20 21:45:07',
  'post_content' => '',
  'post_title' => 'Layouts',
  'post_excerpt' => '',
  'post_name' => 'layouts',
  'post_modified' => '2017-08-21 08:07:48',
  'post_modified_gmt' => '2017-08-21 08:07:48',
  'post_content_filtered' => '',
  'post_parent' => 0,
  'guid' => 'https://themify.me/demo/themes/pinboard/?page_id=178',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_width' => 'default_width',
    'hide_page_title' => 'default',
    'query_category' => '0',
    'order' => 'desc',
    'orderby' => 'date',
    'post_in_lightbox' => 'default',
    'display_content' => 'content',
    'hide_title' => 'default',
    'unlink_title' => 'default',
    'hide_date' => 'default',
    'hide_meta' => 'default',
    'hide_image' => 'default',
    'unlink_image' => 'default',
    'hide_navigation' => 'default',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 181,
  'post_date' => '2012-06-20 21:46:40',
  'post_date_gmt' => '2012-06-20 21:46:40',
  'post_content' => '',
  'post_title' => 'Medium Board',
  'post_excerpt' => '',
  'post_name' => 'medium-board',
  'post_modified' => '2017-10-28 00:29:09',
  'post_modified_gmt' => '2017-10-28 00:29:09',
  'post_content_filtered' => '',
  'post_parent' => 178,
  'guid' => 'https://themify.me/demo/themes/pinboard/?page_id=181',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_width' => 'default_width',
    'hide_page_title' => 'default',
    'query_category' => '0',
    'order' => 'desc',
    'orderby' => 'date',
    'post_in_lightbox' => 'default',
    'layout' => 'grid3',
    'display_content' => 'content',
    'image_width' => '310',
    'hide_title' => 'default',
    'unlink_title' => 'default',
    'hide_date' => 'default',
    'hide_meta' => 'default',
    'hide_image' => 'default',
    'unlink_image' => 'default',
    'hide_navigation' => 'default',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 186,
  'post_date' => '2012-06-20 21:47:19',
  'post_date_gmt' => '2012-06-20 21:47:19',
  'post_content' => '',
  'post_title' => 'Large Board',
  'post_excerpt' => '',
  'post_name' => 'large-board',
  'post_modified' => '2017-08-21 08:07:50',
  'post_modified_gmt' => '2017-08-21 08:07:50',
  'post_content_filtered' => '',
  'post_parent' => 178,
  'guid' => 'https://themify.me/demo/themes/pinboard/?page_id=186',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_width' => 'default_width',
    'hide_page_title' => 'default',
    'query_category' => '0',
    'order' => 'desc',
    'orderby' => 'date',
    'post_in_lightbox' => 'default',
    'layout' => 'grid2',
    'display_content' => 'content',
    'hide_title' => 'default',
    'unlink_title' => 'default',
    'hide_date' => 'default',
    'hide_meta' => 'default',
    'hide_image' => 'default',
    'unlink_image' => 'default',
    'hide_navigation' => 'default',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 189,
  'post_date' => '2012-06-20 21:51:27',
  'post_date_gmt' => '2012-06-20 21:51:27',
  'post_content' => '',
  'post_title' => 'Small Board',
  'post_excerpt' => '',
  'post_name' => 'small-board',
  'post_modified' => '2017-10-27 23:37:50',
  'post_modified_gmt' => '2017-10-27 23:37:50',
  'post_content_filtered' => '',
  'post_parent' => 178,
  'guid' => 'https://themify.me/demo/themes/pinboard/?page_id=189',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'sidebar-none',
    'content_width' => 'default_width',
    'hide_page_title' => 'default',
    'query_category' => '0',
    'order' => 'desc',
    'orderby' => 'date',
    'post_in_lightbox' => 'default',
    'display_content' => 'content',
    'image_width' => '222',
    'hide_title' => 'default',
    'unlink_title' => 'default',
    'hide_date' => 'default',
    'hide_meta' => 'default',
    'hide_image' => 'default',
    'unlink_image' => 'default',
    'hide_navigation' => 'default',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );


$post = array (
  'ID' => 284,
  'post_date' => '2012-12-07 21:23:03',
  'post_date_gmt' => '2012-12-07 21:23:03',
  'post_content' => '',
  'post_title' => 'List Board',
  'post_excerpt' => '',
  'post_name' => 'list-board',
  'post_modified' => '2017-10-28 00:28:39',
  'post_modified_gmt' => '2017-10-28 00:28:39',
  'post_content_filtered' => '',
  'post_parent' => 178,
  'guid' => 'https://themify.me/demo/themes/pinboard/?page_id=284',
  'menu_order' => 0,
  'post_type' => 'page',
  'meta_input' => 
  array (
    'page_layout' => 'default',
    'content_width' => 'default_width',
    'hide_page_title' => 'default',
    'query_category' => '0',
    'order' => 'desc',
    'orderby' => 'date',
    'post_in_lightbox' => 'default',
    'layout' => 'list-post',
    'display_content' => 'content',
    'image_width' => '728',
    'hide_title' => 'default',
    'unlink_title' => 'default',
    'hide_date' => 'default',
    'hide_meta' => 'default',
    'hide_image' => 'default',
    'unlink_image' => 'default',
    'hide_navigation' => 'default',
  ),
  'tax_input' => 
  array (
  ),
);
themify_process_post_import( $post );



function themify_import_get_term_id_from_slug( $slug ) {
	$menu = get_term_by( "slug", $slug, "nav_menu" );
	return is_wp_error( $menu ) ? 0 : (int) $menu->term_id;
}

	$widgets = get_option( "widget_themify-most-commented" );
$widgets[1002] = array (
  'title' => 'Most Commented',
  'show_count' => '3',
  'show_thumb' => 'on',
  'thumb_width' => '50',
  'thumb_height' => '50',
  'show_excerpt' => NULL,
  'excerpt_length' => '55',
  'hide_title' => NULL,
  'show_comment_count' => 'on',
);
update_option( "widget_themify-most-commented", $widgets );

$widgets = get_option( "widget_themify-twitter" );
$widgets[1003] = array (
  'title' => 'Latest Tweets',
  'username' => 'themify',
  'show_count' => '3',
  'hide_timestamp' => NULL,
  'show_follow' => 'on',
  'follow_text' => '→ Follow me',
);
update_option( "widget_themify-twitter", $widgets );

$widgets = get_option( "widget_themify-social-links" );
$widgets[1004] = array (
  'title' => '',
  'show_link_name' => NULL,
  'open_new_window' => NULL,
  'thumb_width' => '',
  'thumb_height' => '',
);
update_option( "widget_themify-social-links", $widgets );



$sidebars_widgets = array (
  'sidebar-main' => 
  array (
    0 => 'themify-most-commented-1002',
    1 => 'themify-twitter-1003',
  ),
  'social-widget' => 
  array (
    0 => 'themify-social-links-1004',
  ),
); 
update_option( "sidebars_widgets", $sidebars_widgets );


	ob_start(); ?>a:64:{s:21:"setting-webfonts_list";s:11:"recommended";s:22:"setting-default_layout";s:12:"sidebar-none";s:27:"setting-default_post_layout";s:5:"grid4";s:30:"setting-default_layout_display";s:7:"content";s:25:"setting-default_more_text";s:4:"More";s:21:"setting-index_orderby";s:4:"date";s:19:"setting-index_order";s:4:"DESC";s:31:"setting-image_post_feature_size";s:5:"blank";s:24:"setting-image_post_width";s:3:"335";s:19:"setting-open_inline";s:2:"on";s:24:"setting-hidesocial_index";s:2:"on";s:32:"setting-default_page_post_layout";s:8:"sidebar1";s:38:"setting-image_post_single_feature_size";s:5:"blank";s:27:"setting-default_page_layout";s:8:"sidebar1";s:53:"setting-customizer_responsive_design_tablet_landscape";s:4:"1024";s:43:"setting-customizer_responsive_design_tablet";s:3:"768";s:43:"setting-customizer_responsive_design_mobile";s:3:"480";s:33:"setting-mobile_menu_trigger_point";s:4:"1200";s:24:"setting-gallery_lightbox";s:8:"lightbox";s:26:"setting-page_builder_cache";s:2:"on";s:27:"setting-page_builder_expiry";s:1:"2";s:19:"setting-entries_nav";s:8:"numbered";s:24:"setting-homepage_welcome";s:174:"<h3>Welcome to Pinboard</h3>
<p>This theme is inspired by the wonderful Pinterest. Resize your browser window or check with a mobile device to see the responsive design.</p>";s:18:"setting-more_posts";s:8:"infinite";s:22:"setting-footer_widgets";s:17:"footerwidget-3col";s:25:"setting-img_php_base_size";s:5:"large";s:27:"setting-global_feature_size";s:5:"large";s:22:"setting-link_icon_type";s:9:"font-icon";s:32:"setting-link_type_themify-link-0";s:10:"image-icon";s:33:"setting-link_title_themify-link-0";s:7:"Twitter";s:32:"setting-link_link_themify-link-0";s:21:"//twitter.com/themify";s:31:"setting-link_img_themify-link-0";s:97:"https://themify.me/demo/themes/pinboard/wp-content/themes/pinboard/themify/img/social/twitter.png";s:32:"setting-link_type_themify-link-1";s:10:"image-icon";s:33:"setting-link_title_themify-link-1";s:8:"Facebook";s:32:"setting-link_link_themify-link-1";s:22:"//facebook.com/themify";s:31:"setting-link_img_themify-link-1";s:98:"https://themify.me/demo/themes/pinboard/wp-content/themes/pinboard/themify/img/social/facebook.png";s:32:"setting-link_type_themify-link-2";s:10:"image-icon";s:33:"setting-link_title_themify-link-2";s:7:"Google+";s:31:"setting-link_img_themify-link-2";s:101:"https://themify.me/demo/themes/pinboard/wp-content/themes/pinboard/themify/img/social/google-plus.png";s:32:"setting-link_type_themify-link-3";s:10:"image-icon";s:33:"setting-link_title_themify-link-3";s:7:"YouTube";s:32:"setting-link_link_themify-link-3";s:32:"//www.youtube.com/user/themifyme";s:31:"setting-link_img_themify-link-3";s:97:"https://themify.me/demo/themes/pinboard/wp-content/themes/pinboard/themify/img/social/youtube.png";s:32:"setting-link_type_themify-link-4";s:10:"image-icon";s:33:"setting-link_title_themify-link-4";s:9:"Pinterest";s:32:"setting-link_link_themify-link-4";s:16:"//pinterest.com/";s:31:"setting-link_img_themify-link-4";s:99:"https://themify.me/demo/themes/pinboard/wp-content/themes/pinboard/themify/img/social/pinterest.png";s:32:"setting-link_type_themify-link-5";s:9:"font-icon";s:33:"setting-link_title_themify-link-5";s:7:"Twitter";s:32:"setting-link_link_themify-link-5";s:21:"//twitter.com/themify";s:33:"setting-link_ficon_themify-link-5";s:10:"fa-twitter";s:32:"setting-link_type_themify-link-6";s:9:"font-icon";s:33:"setting-link_title_themify-link-6";s:8:"Facebook";s:32:"setting-link_link_themify-link-6";s:22:"//facebook.com/themify";s:33:"setting-link_ficon_themify-link-6";s:11:"fa-facebook";s:32:"setting-link_type_themify-link-7";s:9:"font-icon";s:33:"setting-link_title_themify-link-7";s:7:"YouTube";s:32:"setting-link_link_themify-link-7";s:32:"//www.youtube.com/user/themifyme";s:33:"setting-link_ficon_themify-link-7";s:15:"fa-youtube-play";s:22:"setting-link_field_ids";s:273:"{"themify-link-0":"themify-link-0","themify-link-1":"themify-link-1","themify-link-2":"themify-link-2","themify-link-3":"themify-link-3","themify-link-4":"themify-link-4","themify-link-5":"themify-link-5","themify-link-6":"themify-link-6","themify-link-7":"themify-link-7"}";s:23:"setting-link_field_hash";s:1:"8";s:30:"setting-page_builder_is_active";s:6:"enable";s:46:"setting-page_builder_animation_parallax_scroll";s:6:"mobile";s:4:"skin";s:91:"https://themify.me/demo/themes/pinboard/wp-content/themes/pinboard/themify/img/non-skin.gif";}<?php $themify_data = unserialize( ob_get_clean() );

	// fix the weird way "skin" is saved
	if( isset( $themify_data['skin'] ) ) {
		$parsed_skin = parse_url( $themify_data['skin'], PHP_URL_PATH );
		$basedir_skin = basename( dirname( $parsed_skin ) );
		$themify_data['skin'] = trailingslashit( get_template_directory_uri() ) . 'skins/' . $basedir_skin . '/style.css';
	}

	themify_set_data( $themify_data );
	
}
themify_do_demo_import();